@@ -543,12 +543,13 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
 					     NULL, prev->g.c)
 			 != MCHAR_INVALID_CODE))
 		{
-		  start--, prev--;
 		  prev->rface->rfont = this->rface->rfont;
+		  start--, prev--;
 		}
 
 	      for (g++;
 		   (g->type == GLYPH_CHAR
+		    && g->g.from != gstring->control.cursor_pos
 		    && g->rface->layouter == this->rface->layouter
 		    && (g->rface->rfont == this->rface->rfont
 			|| (g->category == GLYPH_CATEGORY_FORMATTER
@@ -1515,6 +1516,7 @@ get_gstring (MFrame *frame, MText *mt, int pos, int to, MDrawControl *control)
 	      || memcmp (control, &gstring->control,
 			 (char *) (&control->with_cursor)
 			 - (char *) (control))
+	      || control->cursor_pos != gstring->control.cursor_pos
 	      || control->cursor_width != gstring->control.cursor_width
 	      || control->cursor_bidi != gstring->control.cursor_bidi)
 	    {
@@ -2831,7 +2833,8 @@ int mdraw_line_break_option;
     character, and incremented each time when a long line is broken
     because of the width limit.
 
-    @return This function returns a character position to break the
+    @return 
+    This function returns a character position to break the
     line.
 */
 
