@@ -35,8 +35,8 @@
     When the key of a font property is @c Msize or @c Mresolution, its
     value is an integer.  Otherwise the value is a symbol.  
 
-    "The font property that belongs to font F and whose key is @c
-    Mxxx" may be shortened to "the xxx property of F".
+    The notation "xxx property of F" means the font property that
+    belongs to font F and whose key is @c Mxxx.
 
     The value of a foundry property is a symbol representing font
     foundry information, e.g. adobe, misc, etc.
@@ -2397,7 +2397,8 @@ mfont_copy (MFont *font)
 
 	#Mfont_ascent, #Mfont_descent, #Mmax_advance.
 
-    @return If $KEY is @c Mfoundry, @c Mfamily, @c Mweight, @c Mstyle,
+    @return 
+    If $KEY is @c Mfoundry, @c Mfamily, @c Mweight, @c Mstyle,
     @c Mstretch, @c Madstyle, @c Mregistry, or @c Mspacing, this
     function returns the corresponding value as a symbol.  If the font
     does not have $KEY property, it returns @c Mnil.  If $KEY is @c
@@ -2417,7 +2418,8 @@ mfont_copy (MFont *font)
 	@c Mfoundry, @c Mfamily, @c Mweight, @c Mstyle, @c Mstretch,
 	@c Madstyle, @c Mregistry, @c Msize, @c Mresolution, @c Mspacing.
 
-    @return $KEY  @c Mfoundry, @c Mfamily, @c Mweight, @c Mstyle, @c
+    @return 
+    $KEY  @c Mfoundry, @c Mfamily, @c Mweight, @c Mstyle, @c
     Mstretch, @c Madstyle, @c Mregistry, @c Mspacing Τ줫ǤС
     ͤ򥷥ܥȤ֤եȤΥץѥƥʤ
     ˤ@c Mnil ֤$KEY  @c Msize 뤤 @c Mresolution 
@@ -3040,7 +3042,8 @@ mfont_list_family_names (MFrame *frame)
     The mfont_check () function checkes if $FONT can be used for
     $SCRIPT and $LANGUAGE in $FONTSET on $FRAME.
 
-    @return If the font is usable, return 1.  Otherwise return 0.
+    @return 
+    If the font is usable, return 1.  Otherwise return 0.
  */
 
 int
