@@ -734,7 +734,8 @@ mscript_list (void)
     $SCRIPT ϥܥǡ̾ Unicode Character Database ˼
     Ƥ륹ץ̾򤹤٤ƾʸˤΤǤ롣
 
-    @return δؿϡ (well-formed) plist ֤ƥ 
+    @return 
+    δؿϡ (well-formed) plist ֤ƥ 
     #Msymbol Ǥꡢġͤ ISO639-1 줿2ʸ쥳
     (Ƥʤ ISO639-2 줿3ʸ쥳) ̾
     Ȥ륷ܥǤ롣֤ plist ѹꤷ
