@@ -46,9 +46,9 @@ M17N_BEGIN_HEADER
 #if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
 
 #define M17NLIB_MAJOR_VERSION 1
-#define M17NLIB_MINOR_VERSION 3
-#define M17NLIB_PATCH_LEVEL 5
-#define M17NLIB_VERSION_NAME "1.3.5"
+#define M17NLIB_MINOR_VERSION 5
+#define M17NLIB_PATCH_LEVEL 2
+#define M17NLIB_VERSION_NAME "1.5.2"
 
 extern void m17n_init_core (void);
 #define M17N_INIT() m17n_init_core ()
@@ -89,12 +89,16 @@ enum M17NStatus
 
 extern enum M17NStatus m17n_status (void);
 
-/***en @defgroup m17nCore CORE API  */
-/***ja @defgroup m17nCore  API */
+/***en @defgroup m17nCore CORE API
+    @brief API provided by libm17n-core.so */
+/***ja @defgroup m17nCore  API
+    @brief libm17n-core.so 󶡤 API */
 /*=*/
 /*** @ingroup m17nCore */
-/***en @defgroup m17nObject Managed Object */
-/***ja @defgroup m17nObject ֥ */
+/***en @defgroup m17nObject Managed Object
+    @brief Objects managed by the reference count */
+/***ja @defgroup m17nObject ֥
+    @brief ȲǴ륪֥ */
 /*=*/
 
 /*** @ingroup m17nObject  */
