@@ -17,7 +17,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the m17n library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    02111-1307, USA.  */
 
 /***en
@@ -237,12 +237,13 @@ static MDeviceLibraryInterface null_interface =
 void
 m17n_init_win (void)
 {
-  int mdebug_mask = MDEBUG_INIT;
+  int mdebug_flag = MDEBUG_INIT;
 
   merror_code = MERROR_NONE;
   if (m17n__gui_initialized++)
     return;
   m17n_init ();
+  m17n_init_flt ();
   if (merror_code != MERROR_NONE)
     {
       m17n__gui_initialized--;
@@ -297,7 +298,7 @@ m17n_init_win (void)
 void
 m17n_fini_win (void)
 {
-  int mdebug_mask = MDEBUG_FINI;
+  int mdebug_flag = MDEBUG_FINI;
   MPlist *plist;
 
   if (m17n__gui_initialized == 0
@@ -341,6 +342,7 @@ m17n_fini_win (void)
   MDEBUG_POP_TIME ();
   MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize the gui modules."));
   MDEBUG_POP_TIME ();
+  m17n_fini_flt ();
   m17n_fini ();
 }
 
@@ -655,13 +657,12 @@ mframe (MPlist *plist)
       if (! interface->handle)
 	{
 	  if (! (interface->handle = dlopen (interface->library, RTLD_NOW))
-	      || ! (interface->init
-		    = (int (*) ()) dlsym (interface->handle, "device_init"))
-	      || ! (interface->open
-		    = (int (*) (MFrame *, MPlist *)) dlsym (interface->handle,
-							    "device_open"))
-	      || ! (interface->fini
-		    = (int (*) ()) dlsym (interface->handle, "device_fini"))
+	      || ! (*(void **) (&interface->init)
+		    = dlsym (interface->handle, "device_init"))
+	      || ! (*(void **) (&interface->open)
+		    = dlsym (interface->handle, "device_open"))
+	      || ! (*(void **) (&interface->fini)
+		    = dlsym (interface->handle, "device_fini"))
 	      || (*interface->init) () < 0)
 	    {
 	      fprintf (stderr, "%s\n", (char *) dlerror ());
