@@ -17,7 +17,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the m17n library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    02111-1307, USA.  */
 
 #ifndef _M17N_H_
@@ -31,23 +31,39 @@
 #include <m17n-core.h>
 #endif
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
+M17N_BEGIN_HEADER
 
 #if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
 extern void m17n_init (void);
 #undef M17N_INIT
+#ifdef _M17N_FLT_H_
+#define M17N_INIT()	\
+  do {			\
+    m17n_init ();	\
+    m17n_init_flt ();	\
+  } while (0)
+#else  /* not _M17N_FLT_H_ */
 #define M17N_INIT() m17n_init ()
+#endif	/* not _M17N_FLT_H_ */
 
 extern void m17n_fini (void);
 #undef M17N_FINI
+#ifdef _M17N_FLT_H_
+#define M17N_FINI()	\
+  do {			\
+    m17n_fini_flt ();	\
+    m17n_fini ();	\
+  } while (0)
+#else  /* not _M17N_FLT_H_ */
 #define M17N_FINI() m17n_fini ()
+#endif	/* not _M17N_FLT_H_ */
+
 #endif
 
-/***en @defgroup m17nShell SHELL API */
-/***ja @defgroup m17nShell  API */
+/***en @defgroup m17nShell SHELL API
+    @brief API provided by libm17n.so */
+/***ja @defgroup m17nShell  API
+    @brief libm17n.so 󶡤 API */
 /*=*/
 
 /*
@@ -55,51 +71,6 @@ extern void m17n_fini (void);
  */
 /*=*/
 
-/*** @ingroup m17nShell */
-/***en @defgroup m17nDatabase Database */
-/***ja @defgroup m17nDatabase ǡ١ */
-/*=*/
-
-/* Directory of an application specific databases.  */
-extern char *mdatabase_dir;
-/*=*/
-/***
-    @ingroup m17nDatabase  */ 
-/***en
-    @brief Type of database.
-
-    The type #MDatabase is for a database object.  Its internal
-    structure is concealed from an application program.  */
-/***ja 
-    @brief ǡ١η.
-
-    #MDatabase ϥǡ١֥Ѥι¤ΤǤ롣
-    ¤ϥץꥱץफϸʤ
-    */
-
-typedef struct MDatabase MDatabase;
-
-/*=*/
-
-/* Look for a data.  */
-extern MDatabase *mdatabase_find (MSymbol tag1, MSymbol tag2,
-				  MSymbol tag3, MSymbol tag4);
-
-extern MPlist *mdatabase_list (MSymbol tag0, MSymbol tag1,
-			       MSymbol tag2, MSymbol tag3);
-
-/* Load a data.  */
-void *mdatabase_load (MDatabase *mdb);
-
-/* Get tags of a data.  */
-extern MSymbol *mdatabase_tag (MDatabase *mdb);
-
-/* Define a data.  */
-extern MDatabase *mdatabase_define (MSymbol tag1, MSymbol tag2,
-				    MSymbol tag3, MSymbol tag4,
-				    void *(*loader) (MSymbol *, void *),
-				    void *extra_info);
-
 /*=*/
 /* (S2) Charset staffs */
 /*=*/
@@ -145,8 +116,6 @@ extern MSymbol Msubset;
 extern MSymbol Msuperset;
 
 /* etc. */
-extern MSymbol Mcharset;
-
 extern MSymbol mchar_define_charset (const char *name, MPlist *plist);
 
 extern MSymbol mchar_resolve_charset (MSymbol symbol);
@@ -431,14 +400,12 @@ enum MCodingType
 	The dimension of each charset defines the length of bytes to
 	represent a single character of the charset, and a byte
 	sequence directly represents the code-point of a character.
-
 	The m17n library provides the default decoding and encoding
 	routines of this type.  */
 
     /***ja
 	ΥפΥɷϤʸåȤľܥݡȤ롣
 	ʸåȤμȤϡʸåȤǰʸɽ뤿ɬפʥХȿǤꡢХʸΥɥݥȤľɽ
-
 	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥɥ롼ƥ󶡤롣  */
 
     MCODING_TYPE_CHARSET,
@@ -446,13 +413,11 @@ enum MCodingType
     /***en
 	A coding system of this type supports byte sequences of a
 	UTF (UTF-8, UTF-16, UTF-32) like structure.
-
 	The m17n library provides the default decoding and encoding
 	routines of this type.  */
 
     /***ja
 	ΥפΥɷϤϡUTF  (UTF-8, UTF-16, UTF-32) ΥХ򥵥ݡȤ롣
-
 	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥɥ롼ƥ󶡤롣  */
 
     MCODING_TYPE_UTF,
@@ -461,14 +426,12 @@ enum MCodingType
 	A coding system of this type supports byte sequences of an
 	ISO-2022 like structure.  The details of each structure are
 	specified by @c MCodingInfoISO2022 .
-
 	The m17n library provides decoding and encoding routines of
 	this type.  */
 
     /***ja 
         ΥפΥɷϤϡISO-2022 ϤΥХ򥵥ݡȤ롣
 	ƥɷϤι¤ξܺ٤ @c MCodingInfoISO2022 ǻꤵ롣
-
 	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥɥ롼ƥ󶡤롣  */
 
     MCODING_TYPE_ISO_2022,
@@ -476,14 +439,12 @@ enum MCodingType
     /***en
 	A coding system of this type is for byte sequences of
 	miscellaneous structures.
-
 	The m17n library does not provide decoding and encoding
 	routines of this type.  They must be provided by the
 	application program.  */
 
     /***ja
 	ΥפΥɷϤϡ¾ι¤ΥХΤΤΤǤ롣
-
 	m17n 饤֥ϤΥѤΥ󥳡ɡǥɥ롼ƥ󶡤ʤΤǡץꥱץ¦ǽɬפ롣  */
 
     MCODING_TYPE_MISC
@@ -847,7 +808,8 @@ extern MPlist *mlanguage_list (void);
 
 extern MSymbol mlanguage_code (MSymbol language, int len);
 
-extern MSymbol mlanguage_name (MSymbol language);
+extern MPlist *mlanguage_name_list (MSymbol language, MSymbol target,
+				    MSymbol script, MSymbol territory);
 
 extern MText *mlanguage_text (MSymbol language);
 
@@ -855,6 +817,8 @@ extern MPlist *mscript_list (void);
 
 extern MPlist *mscript_language_list (MSymbol script);
 
+extern MSymbol mlanguage_name (MSymbol language);
+
 extern MLocale *mlocale_set (int category, const char *locale);
 
 extern MSymbol mlocale_get_prop (MLocale *locale, MSymbol key);
@@ -1397,6 +1361,8 @@ extern char *minput_config_file (void);
 
 extern int minput_save_config (void);
 
+extern int minput_callback (MInputContext *ic, MSymbol command);
+
 /* obsolete functions */
 extern MPlist *minput_get_commands (MSymbol language, MSymbol name);
 
@@ -1412,9 +1378,7 @@ extern int minput_set_variable (MSymbol language, MSymbol name,
 extern MInputMethod *mdebug_dump_im (MInputMethod *im, int indent);
 
 
-#ifdef __cplusplus
-}
-#endif
+M17N_END_HEADER
 
 #endif /* _M17N_H_ */
 
