@@ -483,7 +483,8 @@ msymbol_as_managing_key (const char *name)
     The msymbol_is_managing_key () function checks if the symbol
     $SYMBOL is a managing key or not.
 
-    @return Return 1 if the symbol is a managing key.  Otherwise,
+    @return 
+    Return 1 if the symbol is a managing key.  Otherwise,
     return 0.  */
 
 int
@@ -676,7 +677,8 @@ msymbol_get (MSymbol symbol, MSymbol key)
     ͤϴؿݥ $FUNC Ǥ롣 */
 
 /***
-    @seealso msymbol_put (), M17N_FUNC ()  */
+    @seealso
+     msymbol_put (), M17N_FUNC ()  */
 int
 msymbol_put_func (MSymbol symbol, MSymbol key, M17NFunc func)
 {
